import java.util.Arrays;
import controlP5.*;
import org.processing.wiki.triangulate.*;
import peasy.*;
import org.gicentre.geomap.*;

class PointCount {
    float lon;
    float lat;
    int count;
}

int clamp_range(int min, int max, int val) {
    if (val < min)
        return min;
    else if (val > max)
        return max;
    else
        return val;
}

float clamp_rangef(float min, float max, float val) {
    if (val < min)
        return min;
    else if (val > max)
        return max;
    else
        return val;
}

String makeKey(float val) {
    return "" + Math.round(1.5 * val);
}


// PeasyCam
// PeasyCam(PApplet parent, double lookAtX, double lookAtY, double lookAtZ, double distance);
// PeasyCam(PApplet parent, double distance); // look at 0,0,0
PeasyCam cam;
float[] rotations = new float[3];

ControlP5 cp5;
Accordion accordion;
boolean animation = false;

PImage colorBar;
String[] colorPath = {"plasma.png", "BuRd.png", "magma.png", "viridis.png"};


// --- GEOGRAPHIC VARIABLES ---
GeoMap geoMap = new GeoMap(this);
Table attributes;
java.util.List<Integer> cont_us_IDs = Arrays.asList(1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 33, 34, 38, 39, 40, 43, 44, 45, 46, 47, 48, 49, 51, 52, 53, 55, 56);
ArrayList<ArrayList<PVector>> state_border_points = new ArrayList<ArrayList<PVector>>();
ArrayList<Polygon> state_polygons;


// --- EARTH ---
float EARTH_RADIUS = 25;
float HEIGHT_SCALE = 0.000002;
float NORTH     = 49.92939;
float SOUTH     = 24.116905;
float EAST      = (-61.404427) % 360;
float WEST      = (-128.73395) % 360;
float centerX = fromLatLonAlt((NORTH + SOUTH) / 2, (EAST + WEST) / 2, 0).x; // need this for PeasyCam
float centerY = fromLatLonAlt((NORTH + SOUTH) / 2, (EAST + WEST) / 2, 0).y; // need this for PeasyCam
float centerZ = fromLatLonAlt((NORTH + SOUTH) / 2, (EAST + WEST) / 2, 0).z; // need this for PeasyCam


// --- DATA VARIABLES ---
int minYear = 1990;
int maxYear = 2005;
Table data_table;
PointCount[] geoData;
HashMap<String, PointCount> bucketedData;
ArrayList<PVector> embedded2DCoords;
ArrayList<Triangle> heightMapOnSphere;
ArrayList<PShape> triangles;
ArrayList<Triangle> triangulation;
float maxHeight;

int[] state_colors = new int[54];


void setup() {
    size(1600, 1000, P3D);
    smooth(3);
    textMode(SHAPE); // *** Enable for 3D!
    colorBar = loadImage("magma.png");



    // GeoMap geoMap data from shapefile
    // ArrayList<Polygon> state_polygons
    // ArrayList<ArrayList<PVector>> state_border_points
    setup_geomap();
    
    setup_bucketData();

    setup_triangulation();


    // populate ArrayList<PShape> triangles
    // setup_3d_surface_graphics(heightMapOnSphere);

    setup_cam();
    setup_controls();

    for (int i=0; i < state_colors.length; i++) {
        state_colors[i] = color(random(0, 255), random(0, 255), random(0, 255));
    }

    for (Triangle t : heightMapOnSphere) {
        float h = centroid(t).mag() - EARTH_RADIUS;
        if (maxHeight < h) {
            maxHeight = h;
            // println("new max height: " + maxHeight);
        }
    }

    setup_3d_surface_graphics();

}


// setup state_border_points
// setup state_polygons
void setup_geomap() {
    geoMap.readFile("data/tl_2018_us_state_cont/tl_2018_us_state");   // Read shapefile.
    // geoMap.readFile("data/cb_2017_us_nation_5m/cb_2017_us_nation_5m");

	// geoMap.writeAttributesAsTable(100);

    java.util.Map<java.lang.Integer, Feature> features = geoMap.getFeatures();
    println("");
    println("Setting up US State map.");
    println("There are " + features.size() + " features in the shapefile.");
    state_polygons = new ArrayList<Polygon>();
    for (Feature f : features.values()) {
        if (f.getType() == FeatureType.POLYGON) {
            Polygon poly = (Polygon)f;
            // FILTER
            if (within_bounds(poly.getYCoords()[0], poly.getXCoords()[0]))
                state_polygons.add(poly);
        }
    }
    println("Retained " + state_polygons.size() + " features within continental bounds.");

    for (Polygon statePoly : state_polygons) {
        ArrayList<PVector> border = new ArrayList<PVector>();
        int num_vertices = statePoly.getNumVertices();
        float[] xcoords = new float[num_vertices];
        float[] ycoords = new float[num_vertices];
        xcoords = statePoly.getXCoords();
        ycoords = statePoly.getYCoords();
        for (int i=0; i < num_vertices; i++)
            border.add(fromLatLonAlt(ycoords[i], xcoords[i], 0));
        state_border_points.add(border);
    }
}


// PointCount[] geoData -- from loaded data, make & add PointCount for each row, and edit HashMap
// HashMap<String, PointCount> bucketedData -- keep track of cumulative point counts
// NOTE rounding happens here
void setup_bucketData() {
    // load lightning dataset data
    data_table = loadTable("out_geo_20190312_210319_2010-2011.csv", "header, tsv");
    geoData = new PointCount[data_table.getRowCount()];
    bucketedData = new HashMap<String, PointCount>();
    

    println("Setting up aggregate data.");
    println("Processing " + data_table.getRowCount() + " data points.");
    for (int i=0; i < data_table.getRowCount(); i++) {
        TableRow row = data_table.getRow(i);
        PointCount point = new PointCount();
        point.lon = row.getFloat("lon");
        point.lat = row.getFloat("lat");
        point.count = row.getInt(2);
        geoData[i] = point;

        // put or replace prior value
        String myKey = makeKey(point.lat) + "-" + makeKey(point.lon);
        if (bucketedData.get(myKey) != null) {
            point.count += bucketedData.get(myKey).count;
        }
        bucketedData.put(myKey, point);
    }
    println("created " + bucketedData.size() + " buckets.");
}


// get bucketed data and embed into 2D plane.
// Do triangulation --> ArrayList<Triangle> triangulation
// NOTE split up this function to implement iterative changes to data.
void setup_triangulation() {
    // put bucketed data points in lat/lon plane with z=0, then contruct ArrayList<Triangle> triangulation
    // setup_triangulation();
    int dataPointsOutsideStates = 0;
    // map the bucketedData of lat long to 2D plane 
    embedded2DCoords = new ArrayList<PVector>();
    println("Setting up triangulation. Loading bucketed data to 2D Lat/Lon plane for points inside states");
    for (PointCount entry : bucketedData.values()) {
        // map lat and lon to a 2D plane in 3D space
        // FILTER
        if (within_state(entry.lat, entry.lon)) { // Restrict / filter to only draw inside states
            embedded2DCoords.add(new PVector(entry.lat, entry.lon, 0));
            print("+");
        } else {
            dataPointsOutsideStates++;
        }
    }
    println();
    println("Data points loaded as PVectors into 2D embedding.");
    println("There were " + dataPointsOutsideStates + " points removed outside states.");
    println("There are now " + embedded2DCoords.size() + " points to be triangulated.");

    // triangluate the points in that plane 
    triangulation = Triangulate.triangulate(embedded2DCoords);
    println("we got a triangulation with " + triangulation.size() + " triangles from " + embedded2DCoords.size() + " points.");

    
    // For vertices of triangulation, find corresponding aggregated PointCount
    // If it exists, map to sphere with total count value
    int matches = 0;
    int outliers = 0;
    heightMapOnSphere = new ArrayList<Triangle>();
    for (Triangle t : triangulation) {
        PVector p1 = t.p1;      // vertex 1 of triangle in 2D 
        PVector p2 = t.p2;      // vertex 2 of triangle in 2D
        PVector p3 = t.p3;      // vertex 3 of triangle in 2D
        PVector pt1, pt2, pt3;  // vertices of triangle in 3D
        PVector center = centroid(t);

        PointCount pc1 = bucketedData.get(makeKey(p1.x) + "-" + makeKey(p1.y));
        if (pc1 != null) {
            matches++;
            pt1 = fromLatLonAlt(p1.x, p1.y, pc1.count);
        } else {
            println("ERROR! No match found!");
            continue;
        }

        PointCount pc2 = bucketedData.get(makeKey(p2.x) + "-" + makeKey(p2.y));
        if (pc2 != null) {
            matches++;
            pt2 = fromLatLonAlt(p2.x, p2.y, pc2.count);
        } else {
            println("ERROR! No match found!");
            continue;
        }

        PointCount pc3 = bucketedData.get(makeKey(p3.x) + "-" + makeKey(p3.y));
        if (pc3 != null) {
            matches++;
            pt3 = fromLatLonAlt(p3.x, p3.y, pc3.count);
        } else {
            println("ERROR! No match found!");
            continue;
        }
        
        // FILTER
        if (within_state(center.x, center.y)) {
            heightMapOnSphere.add(new Triangle(pt1, pt2, pt3));
        } else {
            outliers++;
        }
    }
    println("Triangulation finished. Checking how many of the triangulated points exist in the buckedData:");
    println("matches: " + matches + " =?= " + 3 * triangulation.size());
    println("There were " + outliers + " triangles removed with centers outside states.");
    println("heightMap + outliers vs orig. triangulation: " + (heightMapOnSphere.size() + outliers) + " =?= " + triangulation.size());
}

void setup_3d_surface_graphics() {
    triangles = new ArrayList<PShape>();
    for (Triangle t : heightMapOnSphere) {
        PShape s = createShape();
        s.beginShape();
        s.fill(computeTriangleColor(t));
        s.noStroke();
        s.strokeWeight(1);
        s.vertex(t.p1.x, t.p1.y, t.p1.z);
        s.vertex(t.p2.x, t.p2.y, t.p2.z);
        s.vertex(t.p3.x, t.p3.y, t.p3.z);
        s.endShape(CLOSE);
        triangles.add(s);
    }
}

void setup_cam() {
    cam = new PeasyCam(this, centerX, - centerY, centerZ, EARTH_RADIUS/2);
    float cameraZ = (height/2.0) / tan(PI*60.0/360.0);
    perspective(PI/2, width/height, 0.001, cameraZ*10.0);
    cam.setWheelScale(0.5);
}


void setup_controls() {
    cp5 = new ControlP5(this);
    int numTickMarks = maxYear - minYear + 1;

    Group g1 = cp5.addGroup("Time Options")
        .setBackgroundColor(color(0, 64))
        .setBackgroundHeight(180);
    
    cp5.addToggle("Animation")
        .setPosition(10,80)
        .setSize(40,40)
        .moveTo(g1);
    
    cp5.addSlider("Start")
        .setPosition(10,20)
        .setSize(140,20)
        .setRange(minYear, maxYear)
        .setValue(2000)
        .setNumberOfTickMarks(numTickMarks)
        .moveTo(g1);
    
    cp5.addSlider("End")
        .setPosition(10,50)
        .setSize(140,20)
        .setRange(minYear, maxYear)
        .setValue(2000)
        .setNumberOfTickMarks(numTickMarks)
        .moveTo(g1);
    
    g1.getController("Start").getValueLabel().setText("" + (int)(g1.getController("Start").getValue()));
    g1.getController("End").getValueLabel().setText("" + (int)(g1.getController("End").getValue()));
    


    Group g2 = cp5.addGroup("Data Options")
        .setBackgroundColor(color(0, 64))
        .setBackgroundHeight(150);
    
    cp5.addRadioButton("radio1")
        .setPosition(10, 20)
        .setItemWidth(20)
        .setItemHeight(20)
        .addItem("A1", 0)
        .addItem("B1", 1)
        .addItem("C1", 2)
        .addItem("D1", 3)
        .setColorLabel(color(255))
        .activate(2)
        .moveTo(g2);
    
    cp5.addRadioButton("radio2")
        .setPosition(80, 20)
        .setItemWidth(20)
        .setItemHeight(20)
        .addItem("A2", 0)
        .addItem("B2", 1)
        .addItem("C2", 2)
        .addItem("D2", 3)
        .setColorLabel(color(255))
        .activate(2)
        .moveTo(g2);

    accordion = cp5.addAccordion("acc")
                 .setPosition(40,40)
                 .setWidth(200)
                 .addItem(g1)
                 .addItem(g2);
    
    accordion.open(0,1);
    accordion.setCollapseMode(Accordion.MULTI);

    cp5.setAutoDraw(false);
}

// *** Actions to perform when controls are changed ***
void radio1(int theChar) {
    switch (theChar) {
        case('1'): 
    }
}

void radio2(int theChar) {
    // 
}

/* some colors:
    #1C9099     ocean
    (32,36,45)  gRGB dark blue
    (43,48,59)  native dark blue
    */


void draw() {
    randomSeed(0);
    background(color(32, 36, 45));
    // stroke(0,40);
    scale(1, -1, 1); // Convert to Right Handed Coordinate System

    draw_states();

    // draw_data();

    // draw_sphere();
    
    // draw_basis(EARTH_RADIUS / 2);

    draw_heightmap();

    draw_controls();


}

// Bounding box in lat-lon coords around continental US
// void draw_rect() {
    //     PVector vec1, vec2;
    //     for (int i=0; i < 50; i++) {
    //         float u1 = (WEST) + ((1.0 * i) / 50) * (EAST - WEST);
    //         float u2 = (WEST) + ((1.0 * (i+1)) / 50) * (EAST - WEST);
    //         strokeWeight(5);
    //         stroke(0);
    //         vec1 = fromLatLonAlt(NORTH, u1, 0);
    //         vec2 = fromLatLonAlt(NORTH, u2, 0);
    //         line(vec1.x, vec1.y, vec1.z, vec2.x, vec2.y, vec2.z);

    //         vec1 = fromLatLonAlt(SOUTH, u1, 0);        
    //         vec2 = fromLatLonAlt(SOUTH, u2, 0);
    //         line(vec1.x, vec1.y, vec1.z, vec2.x, vec2.y, vec2.z);
    //     }

    //     for (int i=0; i < 50; i++) {
    //         float u1 = (SOUTH) + ((1.0 * i) / 50) * (NORTH - SOUTH);
    //         float u2 = (SOUTH) + ((1.0 * (i+1)) / 50) * (NORTH - SOUTH);
    //         strokeWeight(5);
    //         stroke(0);
    //         vec1 = fromLatLonAlt(u1, EAST, 0);
    //         vec2 = fromLatLonAlt(u2, EAST, 0);
    //         line(vec1.x, vec1.y, vec1.z, vec2.x, vec2.y, vec2.z);

    //         vec1 = fromLatLonAlt(u1, WEST, 0);        
    //         vec2 = fromLatLonAlt(u2, WEST, 0);
    //         line(vec1.x, vec1.y, vec1.z, vec2.x, vec2.y, vec2.z);
    //     }
// }

// Sphere representing Earth
void draw_sphere() {
    stroke(255);
    sphereDetail(60);
    noFill();
    sphere(EARTH_RADIUS);
}

// experimenting with coordinate system
// void draw_coords() {
    //     for (int i=70; i < 130; i += 10) {
    //         PVector vNorth = fromLatLonAlt(NORTH, -i, 0);
    //         PVector vSouth = fromLatLonAlt(SOUTH, -i, 0);
    //         textSize(10);
    //         strokeWeight(20);
    //         stroke(255, 0, 0);
    //         point(vNorth.x, vNorth.y, vNorth.z);
    //         stroke(0, 0, 255);
    //         point(vSouth.x, vSouth.y, vSouth.z);
    //     }
// }


void draw_states() {
    // state borders
    for (ArrayList<PVector> poly : state_border_points) {
        for (PVector p : poly) {
            stroke(255);
            strokeWeight(2);
            if (random(0, 1) < 0.08) {
                point(p.x, p.y, p.z);
            }
        }
    }
}

color computeTriangleColor(Triangle t) {
    float h = centroid(t).mag() - EARTH_RADIUS;
    color c = colorBar.get((int)map(clamp_rangef(0, maxHeight, h), 0, maxHeight, 2, colorBar.width-2), colorBar.height/2);
    int alpha = 180;
    return color(c, alpha);
    // int stateID = find_state(centroid(t));
    // if (stateID >= 0) {
    //     color c2 = color(state_colors[stateID]);
    //     PVector colorvec = new PVector(red(c2), green(c2), blue(c2));
    //     colorvec.mult(map(h, 0.0, maxHeight, maxVal(colorvec), 255.0) / maxVal(colorvec));
    //     colorvec.setMag(map(h, 0.0, maxHeight, 85.0 * (float)Math.sqrt(3.0), 255.0 * (float)Math.sqrt(3.0)));
    //     return color(colorvec.x, colorvec.y, colorvec.z, alpha);
    // }
    // return color(255);
}

float L1Norm(PVector p) {
    return abs(p.x) + abs(p.y) + abs(p.z);
}
float maxVal(PVector p) {
    return max(abs(p.x), max(abs(p.y), abs(p.z)));
}

void draw_data() {
    for (PointCount point : geoData) {
        PVector surface = fromLatLonAlt(point.lat, point.lon, 0);
        PVector upper = fromLatLonAlt(point.lat, point.lon, point.count * HEIGHT_SCALE);
        if (random(0,1) < 0.1) {
            // point(surface.x, surface.y, surface.z);
            stroke(color(10, 20, 64, 128));
            strokeWeight(2);
            line(surface.x, surface.y, surface.z, upper.x, upper.y, upper.z);
            
            strokeWeight(8);
            stroke(color(255, 255, 255, 128));
            point(surface.x, surface.y, surface.z);
        }
    }
}


void draw_heightmap() {
    for (PShape s : triangles) {
        shape(s);
    }
}

void draw_controls() {
    cam.beginHUD();
    cp5.draw();
    cam.endHUD();
}

PVector fromLatLonAlt(float lat, float lon, float alt) {
    float x = EARTH_RADIUS * (1 + HEIGHT_SCALE * alt) * cos(lat * PI / 180) * cos(lon * PI / 180);
    float y = EARTH_RADIUS * (1 + HEIGHT_SCALE * alt) * cos(lat * PI / 180) * sin(lon * PI / 180);
    float z = EARTH_RADIUS * (1 + HEIGHT_SCALE * alt) * sin(lat * PI / 180);
    return new PVector(x, y, z);
}

PVector fromLatLonAlt(PVector latlonalt) {
    return fromLatLonAlt(latlonalt.x, latlonalt.y, latlonalt.z);
}

void draw_basis(float sc) {
    shapeMode(CENTER);
    pushMatrix();
    noStroke();
    fill(#FFFFFF, 128);
    sphere(2);
    
    pushMatrix();
    translate(sc, 0, 0);
    fill(#FF0000, 128);
    box(2*sc, sc/8, sc/8);
    popMatrix();
    
    pushMatrix();
    translate(0, sc, 0);
    fill(#00FF00, 128);
    box(sc/8, 2*sc, sc/8);
    popMatrix();

    pushMatrix();
    translate(0, 0, sc);
    fill(#0000FF, 128);
    box(sc/8, sc/8, 2*sc);
    popMatrix();
    
    popMatrix();
}

float cross_measure(Triangle t) {
    return abs(sin(PVector.angleBetween(t.p1.sub(t.p2), t.p1.sub(t.p3))));
}

boolean within_state(float lat, float lon) {
    if (!(within_bounds(lat, lon)))
        return false;
    if (state_polygons.size() == 0)
        return true;
    for (Polygon p : state_polygons) {
        if (p.contains(lon, lat)) {
            return true;
        }
    }
    return false;
}

int find_state(PVector p) {
    PVector v = p.copy().setMag(EARTH_RADIUS);
    PVector u = new PVector(v.x, v.y, 0);
    float lat = atan2(v.z, u.mag()) * 180.0 / PI;
    float lon = atan2(v.y, v.x) * 180.0 / PI;
    for (int i=0; i < state_polygons.size(); i++) {
        Polygon state = state_polygons.get(i);
        if (state.contains(lon, lat)) {
            return i;
        }
    }
    return -1;    
}

// NOTE: calls to this function often of the form within_bounds(Y, X) because
// lat/lon geographic coordinates are stored in shapefile as (x, y) = (lon, lat)
boolean within_bounds(float lat, float lon) {
    return inInterval(lon % 360, WEST, EAST) && inInterval((lat + 90) % 180 - 90, SOUTH, NORTH);
}

PVector centroid(Triangle t) {
    PVector p1 = t.p1.copy();
    PVector p2 = t.p2.copy();
    PVector p3 = t.p3.copy();
    return p1.add(p2).add(p3).mult(1.0 / 3.0);
}

boolean inInterval(float x, float min, float max) {
    return (min <= x) && (x <= max);
}